package Apache::Admin::Config::Expand;
use parent 'Apache::Admin::Config';
use strict;
use warnings;
use Carp;

our $VERSION = '1.0';

sub new {
    my $class = shift;
    my $server_root = Apache::Admin::Config::Tree::_get_arg(\@_,
							    '-server_root');
    my $explist = Apache::Admin::Config::Tree::_get_arg(\@_, '-expand')
	|| [ qw(include) ];
    if (Apache::Admin::Config::Tree::_get_arg(\@_, '-compact!')) {
	push @$explist, 'compact'
    }

    my $self = $class->SUPER::new(@_) or return;
    bless $self, $class;
    $self->{_options} = \@_;
    $self->{_server_root} = $server_root;

    eval {
	return unless $self->_preproc($explist);
    };
    if ($@) {
	$Apache::Admin::Config::ERROR = $@;
	return;
    }
    
    return $self;
}

sub server_root {
    my $self = shift;
    unless ($self->{_server_root}) {
    	if (my $d = $self->directive('ServerRoot')) {
	    $self->{_server_root} = $self->dequote($d->value);
	}
    }
    return $self->{_server_root};
}

sub dequote {
    my ($self, $str) = @_;
    if ($str =~ s/^"(.*)"$/$1/) {
	$str =~ s/\\"/"/g;
    }
    return $str;
}

sub options { shift->{_options} }

sub _preproc {
    my ($self, $explist) = @_;

    return 1 unless @$explist;
    
    return $self->_preproc_section($self,
				   [ map {
				       my $mod = 'Apache::Admin::Config::Expand::'.$_;
				       (my $file = $mod) =~ s|::|/|g;
				       require $file . '.pm';
				       $mod->new
				     } @$explist ]);
}

sub _preproc_section {
    my ($self, $section, $modlist) = @_;

  OUTER:
    for (my $i = 0; defined(my $d = $section->select(-which => $i)); ) {
	foreach my $mod (@$modlist) {
	    my @repl;
	    if ($mod->expand($self, $d, \@repl)) {
		my $prev = $d;
		foreach my $r (@repl) {
		    $prev = $section->add($r, -after => $prev);
		}
		$d->unlink;
		next OUTER;
	    }
	    if ($d->type eq 'section') {
		$self->_preproc_section($d, $modlist);
	    }
	}
	$i++;
    }
    return 1;
}


1;
__END__

=head1 NAME

Apache::Admin::Config::Expand - Preprocess Apache configuration files

=head1 SYNOPSIS

    use Apache::Admin::Config::Expand;
    
    $x = new Apache::Admin::Config::Expand '/path/to/httpd.conf', '-compact'
             or die $Apache::Admin::Config::ERROR;

=head1 DESCRIPTION

B<Apache::Admin::Config::Expand> reads and parses Apache configuration
file, expanding any B<Include> and B<IncludeOptional> statements. Optionally,
whitespace and comments are eliminated. Apart from this, it provides the
same functionality as B<Apache::Admin::Config>, from which it inherits all
methods.

=head1 METHODS

=head2 new

    $obj = new Apache::Admin::Config::Expand [/path/to/file|handle],
          [-compact],
          [-indent => $integer], ['-create'], ['-no-comment-grouping'],
          ['-no-blank-grouping']

Reads an Apache configuration file. The B<-compact> argument, if specified
causes any comment and empty lines to be removed from the configuration.
Rest of arguments and flags have the same meaning as in
B<Apache::Admin::Config>.

Rest of methods is inherited from B<Apache::Admin::Config>.
    
=head1 EXAMPLE

Print on standard output only statements from the configuration file

    use Apache::Admin::Config::Expand;
    $conf = new Apache::Admin::Config::Expand '/etc/http/httpd.conf',
                                              '-compact'
              or die $Apache::Admin::Config::ERROR;
    print $conf->dump_raw;

=cut
    
    
    
